/*******************************************************************************
 * Copyright (C) 2017 Philipp Spohr
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
package de.hhu.ba.yoshikoWrapper.swing.components;

import static javax.swing.GroupLayout.DEFAULT_SIZE;
import static javax.swing.GroupLayout.PREFERRED_SIZE;

import java.awt.ScrollPane;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Comparator;

import javax.swing.BoxLayout;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import org.cytoscape.work.TaskIterator;

import de.hhu.ba.yoshikoWrapper.core.CyCore;
import de.hhu.ba.yoshikoWrapper.core.LocalizationManager;
import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoCluster;
import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoSolution;
import de.hhu.ba.yoshikoWrapper.swing.SwingUtil;
import de.hhu.ba.yoshikoWrapper.tasks.CreateMetaGraphTask;

@SuppressWarnings("serial")
public class SolutionTab extends JPanel {

	private final JScrollPane scrollPane;
	private final ClusterViewList scrollPaneContent;
	private final JLabel clusterCount;
	private final JButton createMetaGraph;

	private final YoshikoSolution solution;

	public SolutionTab(YoshikoSolution s) throws Exception {

		this.solution = s;

		//init swing components
		scrollPaneContent = new ClusterViewList();
		scrollPaneContent.setLayout(new BoxLayout(scrollPaneContent,BoxLayout.Y_AXIS));

		//Sort cluster by size, descending
		solution.cluster.sort(new Comparator<YoshikoCluster>() {

			@Override
			public int compare(YoshikoCluster o1, YoshikoCluster o2) {
				if (o1.getSize() == o2.getSize()) {
					return 0;
				}
				else if(o1.getSize() < o2.getSize()) {
					return 1;
				}
				return -1;
			}

		});

		for (YoshikoCluster c: solution.cluster) {
			ClusterView clusterView = new ClusterView(c);
			scrollPaneContent.add(clusterView);
		}

		scrollPane = new JScrollPane(scrollPaneContent);

		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

		clusterCount = new JLabel(LocalizationManager.get("clusterFound")+" "+s.cluster.size());

		createMetaGraph = new JButton(LocalizationManager.get("createMetaGraph"));
		createMetaGraph.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				CyCore.dialogTaskManager.execute(
						new TaskIterator(1,
								new CreateMetaGraphTask(solution)
						)
				);
			}

		});
		SwingUtil.addAll(this,clusterCount,scrollPane,createMetaGraph);

		//Layout
		GroupLayout layout = new GroupLayout(this);
		layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING,true)
				.addComponent(clusterCount,DEFAULT_SIZE,DEFAULT_SIZE,DEFAULT_SIZE)
				.addComponent(scrollPane,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(createMetaGraph,DEFAULT_SIZE,DEFAULT_SIZE,DEFAULT_SIZE)
		);
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addComponent(clusterCount,DEFAULT_SIZE,DEFAULT_SIZE,PREFERRED_SIZE)
				.addComponent(scrollPane,DEFAULT_SIZE,DEFAULT_SIZE,PREFERRED_SIZE)
				.addComponent(createMetaGraph,DEFAULT_SIZE,DEFAULT_SIZE,PREFERRED_SIZE)
		);

		this.setLayout(layout);
	}

}
